@@ -16,8 +16,10 @@ from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
 from xpra.codecs.loader import get_codec
 from xpra.codecs.video_helper import getVideoHelper
 from xpra.os_util import BytesIOClass, bytestostr, _buffer
-from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
-from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
+try:
+    from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
+except ImportError:
+    from xpra.util import xor as xor_str
 
 DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
 INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
@@ -154,11 +156,13 @@ class WindowBackingBase(object):
         if not self._alpha_enabled:
             target_rgb_modes = tuple(x for x in target_rgb_modes if x.find("A")<0)
         full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*target_rgb_modes)
+        full_csc_modes["webp"] = [x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA")]
         log("_get_full_csc_modes(%s)=%s (target_rgb_modes=%s)", rgb_modes, full_csc_modes, target_rgb_modes)
         return full_csc_modes
 
 
     def unpremultiply(self, img_data):
+        from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
         if type(img_data) not in (str, _buffer):
             try:
                 unpremultiply_argb_in_place(img_data)
@@ -296,8 +300,7 @@ class WindowBackingBase(object):
             return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
         rgb_format = options.strget("rgb_format")
         has_alpha = options.boolget("has_alpha", False)
-        #log("paint_webp%s rgb_format=%s, has_alpha=%s, RGB_MODES=%s", (len(img_data), x, y, width, height, options, callbacks), rgb_format, has_alpha, self.RGB_MODES)
-        buffer_wrapper, width, height, stride, has_alpha, rgb_format = self.webp_decoder.decompress(img_data, has_alpha, rgb_format)
+        buffer_wrapper, width, height, stride, has_alpha, rgb_format = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
         def free_buffer(*_args):
             buffer_wrapper.free()
         callbacks.append(free_buffer)
@@ -312,6 +315,7 @@ class WindowBackingBase(object):
             rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
             rgb_format = img.get_pixel_format()
             data = img.get_pixels()
+            stride = img.get_rowstride()
         #replace with the actual rgb format we get from the decoder:
         options[b"rgb_format"] = rgb_format
         return self.paint_rgb(rgb_format, data, x, y, width, height, stride, options, callbacks)
@@ -359,6 +363,13 @@ class WindowBackingBase(object):
         raise Exception("override me!")
 
 
+    def eos(self):
+        dl = self._decoder_lock
+        with dl:
+            self.do_clean_csc_decoder()
+            self.do_clean_video_decoder()
+
+
     def make_csc(self, src_width, src_height, src_format,
                        dst_width, dst_height, dst_format_options, speed):
         global CSC_OPTIONS
